Index: LinuxBIOSv2-2520/src/superio/ite/it8716f/it8716f_early_init.c
===================================================================
--- /dev/null
+++ LinuxBIOSv2-2520/src/superio/ite/it8716f/it8716f_early_init.c
@@ -0,0 +1,38 @@
+/*
+ * This file is part of the LinuxBIOS project.
+ *
+ * Copyright (C) 2007 AMD
+ * Written by Yinghai Lu <yinghai.lu@amd.com> for AMD.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+ */
+
+#include <arch/romcc_io.h>
+#include "it8716f.h"
+
+static void it8716f_disable_dev(device_t dev)
+{
+	pnp_set_logical_device(dev);
+	pnp_set_enable(dev, 0);
+}
+
+static void it8716f_enable_dev(device_t dev, unsigned iobase)
+{
+	pnp_set_logical_device(dev);
+	pnp_set_enable(dev, 0);
+	pnp_set_iobase(dev, PNP_IDX_IO0, iobase);
+	pnp_set_enable(dev, 1);
+}
+
Index: LinuxBIOSv2-2520/src/superio/ite/it8716f/it8716f_early_serial.c
===================================================================
--- LinuxBIOSv2-2520.orig/src/superio/ite/it8716f/it8716f_early_serial.c
+++ LinuxBIOSv2-2520/src/superio/ite/it8716f/it8716f_early_serial.c
@@ -3,6 +3,9 @@
  *
  * Copyright (C) 2006 Uwe Hermann <uwe@hermann-uwe.de>
  *
+ * Copyright (C) 2007 AMD
+ * Written by Yinghai Lu <yinghai.lu@amd.com> for AMD.
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -35,55 +38,34 @@
 
 #define IT8716F_CONFIGURATION_PORT   0x2e /* Write-only. */
 
-/* The content of IT8716F_CONFIG_REG_LDN (index 0x07) must be set to the
-   LDN the register belongs to, before you can access the register. */
-static void it8716f_sio_write(uint8_t ldn, uint8_t index, uint8_t value)
+/* Perform MB PnP setup to put the SIO chip at 0x2e. */
+/* Base address 0x2e: 0x87 0x01 0x55 0x55. */
+/* Base address 0x4e: 0x87 0x01 0x55 0xaa. */
+static inline void pnp_enter_ext_func_mode(device_t dev) 
 {
-	outb(IT8716F_CONFIG_REG_LDN, SIO_BASE);
-	outb(ldn, SIO_DATA);
-	outb(index, SIO_BASE);
-	outb(value, SIO_DATA);
+	unsigned port = dev >> 8;
+	outb(0x87, port);
+	outb(0x01, port);
+	outb(0x55, port);
+	if (port == 0x4e) {
+		outb(0xaa, port);
+	} else {
+		outb(0x55, port);
+	}
 }
 
-/* Enable the peripheral devices on the IT8716F Super I/O chip. */
-static void it8716f_enable_serial(device_t dev, unsigned iobase)
+static void pnp_exit_ext_func_mode(device_t dev)
 {
-	/* (1) Enter the configuration state (MB PnP mode). */
-
-	/* Perform MB PnP setup to put the SIO chip at 0x2e. */
-	/* Base address 0x2e: 0x87 0x01 0x55 0x55. */
-	/* Base address 0x4e: 0x87 0x01 0x55 0xaa. */
-	outb(0x87, IT8716F_CONFIGURATION_PORT);
-	outb(0x01, IT8716F_CONFIGURATION_PORT);
-	outb(0x55, IT8716F_CONFIGURATION_PORT);
-	outb(0x55, IT8716F_CONFIGURATION_PORT);
-
-	/* (2) Modify the data of configuration registers. */
-
-	/* Select the chip to configure (if there's more than one).
-           Set bit 7 to select JP3=1, clear bit 7 to select JP3=0.
-           If this register is not written, both chips are configured. */
-	/* it8716f_sio_write(0x00, IT8716F_CONFIG_REG_CONFIGSEL, 0x00); */
-
-	/* Enable all devices. */
-	it8716f_sio_write(IT8716F_FDC,  0x30, 0x1); /* Floppy */
-	it8716f_sio_write(IT8716F_SP1,  0x30, 0x1); /* Serial port 1 */
-	it8716f_sio_write(IT8716F_SP2,  0x30, 0x1); /* Serial port 2 */
-	it8716f_sio_write(IT8716F_PP,   0x30, 0x1); /* Parallel port */
-	it8716f_sio_write(IT8716F_EC,   0x30, 0x1); /* Environment controller */
-	it8716f_sio_write(IT8716F_KBCK, 0x30, 0x1); /* Keyboard */
-	it8716f_sio_write(IT8716F_KBCM, 0x30, 0x1); /* Mouse */
-	it8716f_sio_write(IT8716F_MIDI, 0x30, 0x1); /* MIDI port */
-	it8716f_sio_write(IT8716F_GAME, 0x30, 0x1); /* GAME port */
-	it8716f_sio_write(IT8716F_IR,   0x30, 0x1); /* Consumer IR */
-
-	/* Select 24MHz CLKIN (set bit 0). */
-	it8716f_sio_write(0x00, IT8716F_CONFIG_REG_CLOCKSEL, 0x01);
-
-	/* Clear software suspend mode (clear bit 0). TODO: Needed? */
-	/* it8716f_sio_write(0x00, IT8716F_CONFIG_REG_SWSUSP, 0x00); */
+	pnp_write_config(dev, 0x02, 0x02);
+}
 
-	/* (3) Exit the configuration state (MB PnP mode). */
-	it8716f_sio_write(0x00, IT8716F_CONFIG_REG_CC, 0x02);
+static void it8716f_enable_serial(device_t dev, unsigned iobase)
+{
+	pnp_enter_ext_func_mode(dev);
+	pnp_set_logical_device(dev);
+	pnp_set_enable(dev, 0);
+	pnp_set_iobase(dev, PNP_IDX_IO0, iobase);
+	pnp_set_enable(dev, 1);
+	pnp_exit_ext_func_mode(dev);
 }
 
Index: LinuxBIOSv2-2520/src/superio/ite/it8716f/superio.c
===================================================================
--- LinuxBIOSv2-2520.orig/src/superio/ite/it8716f/superio.c
+++ LinuxBIOSv2-2520/src/superio/ite/it8716f/superio.c
@@ -3,6 +3,9 @@
  *
  * Copyright (C) 2006 Uwe Hermann <uwe@hermann-uwe.de>
  *
+ * Copyright (C) 2007 AMD
+ * Written by Yinghai Lu <yinghai.lu@amd.com> for AMD.
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -24,8 +27,29 @@
 #include <pc80/keyboard.h>
 #include "chip.h"
 #include "it8716f.h"
+#include <arch/io.h>
+
+/* Base address 0x2e: 0x87 0x01 0x55 0x55. */
+/* Base address 0x4e: 0x87 0x01 0x55 0xaa. */
+static void pnp_enter_ext_func_mode(device_t dev)
+{
+	outb(0x87, dev->path.u.pnp.port);
+	outb(0x01, dev->path.u.pnp.port);
+	outb(0x55, dev->path.u.pnp.port);
+
+	if (dev->path.u.pnp.port == 0x4e) {
+		outb(0xaa, dev->path.u.pnp.port);
+	} else {
+		outb(0x55, dev->path.u.pnp.port);
+	}
+}
+
+static void pnp_exit_ext_func_mode(device_t dev)
+{
+	pnp_write_config(dev, 0x02, 0x02);
+}
 
-static void init(device_t dev)
+static void it8716f_init(device_t dev)
 {
 	struct superio_ite_it8716f_config *conf;
 	struct resource *res0, *res1;
@@ -36,9 +60,8 @@ static void init(device_t dev)
 
 	conf = dev->chip_info;
 
+	/* TODO: FDC, PP, EC, KBCM, MIDI, GAME, IR. */
 	switch (dev->path.u.pnp.device) {
-	case IT8716F_FDC: /* TODO. */
-		break;
 	case IT8716F_SP1:
 		res0 = find_resource(dev, PNP_IDX_IO0);
 		init_uart8250(res0->base, &conf->com1);
@@ -47,44 +70,61 @@ static void init(device_t dev)
 		res0 = find_resource(dev, PNP_IDX_IO0);
 		init_uart8250(res0->base, &conf->com2);
 		break;
-	case IT8716F_PP: /* TODO. */
-		break;
-	case IT8716F_EC: /* TODO. */
-		break;
 	case IT8716F_KBCK:
 		res0 = find_resource(dev, PNP_IDX_IO0);
 		res1 = find_resource(dev, PNP_IDX_IO1);
 		init_pc_keyboard(res0->base, res1->base, &conf->keyboard);
 		break;
-	case IT8716F_KBCM: /* TODO. */
-		break;
-	case IT8716F_MIDI: /* TODO. */
-		break;
-	case IT8716F_GAME: /* TODO. */
-		break;
-	case IT8716F_IR: /* TODO. */
-		break;
 	}
 }
 
+static void it8716f_pnp_set_resources(device_t dev)
+{
+	pnp_enter_ext_func_mode(dev);
+	pnp_set_resources(dev);
+	pnp_exit_ext_func_mode(dev);
+}
+
+static void it8716f_pnp_enable_resources(device_t dev)
+{
+	pnp_enter_ext_func_mode(dev);
+	pnp_enable_resources(dev);
+	pnp_exit_ext_func_mode(dev);
+}
+
+static void it8716f_pnp_enable(device_t dev)
+{
+	pnp_enter_ext_func_mode(dev);
+	pnp_set_logical_device(dev);
+	pnp_set_enable(dev, dev->enabled);
+	pnp_exit_ext_func_mode(dev);
+}
+
 static struct device_operations ops = {
 	.read_resources   = pnp_read_resources,
-	.set_resources    = pnp_set_resources,
-	.enable_resources = pnp_enable_resources,
-	.enable           = pnp_enable,
-	.init             = init,
+	.set_resources    = it8716f_pnp_set_resources,
+	.enable_resources = it8716f_pnp_enable_resources,
+	.enable           = it8716f_pnp_enable,
+	.init             = it8716f_init,
 };
 
-/* TODO: FDC, PP, EC, KBCM, MIDI, GAME, IR. */
 static struct pnp_info pnp_dev_info[] = {
- { &ops, IT8716F_SP1,  PNP_IO0 | PNP_IRQ0, { 0x7f8, 0 }, },
- { &ops, IT8716F_SP2,  PNP_IO0 | PNP_IRQ0 | PNP_DRQ0 | PNP_DRQ1, { 0x7f8, 0 }, },
- { &ops, IT8716F_KBCK, PNP_IO0 | PNP_IO1 | PNP_IRQ0, { 0x7f8, 0 }, { 0x7f8, 0x4}, },
+	{ &ops, IT8716F_FDC, PNP_IO0 | PNP_IRQ0 | PNP_DRQ0, { 0x07f8, 0}, },
+	{ &ops, IT8716F_SP1, PNP_IO0 | PNP_IRQ0, { 0x7f8, 0 }, },
+	{ &ops, IT8716F_SP2, PNP_IO0 | PNP_IRQ0, { 0x7f8, 0 }, },
+	{ &ops, IT8716F_PP, PNP_IO0 | PNP_IRQ0 | PNP_DRQ0, { 0x07f8, 0}, },
+	{ &ops, IT8716F_EC, },
+	{ &ops, IT8716F_KBCK, PNP_IO0 | PNP_IO1 | PNP_IRQ0, { 0x7ff, 0 }, { 0x7ff, 0x4}, },
+	{ &ops, IT8716F_KBCM, PNP_IRQ0, },
+	// No 7 { 0,},
+	{ &ops, IT8716F_MIDI, PNP_IO0 | PNP_IRQ0, {0x7fe, 0x4}, },
+	{ &ops, IT8716F_GAME, PNP_IO0, { 0x7ff, 0 }, },
+	{ &ops, IT8716F_IR, },
 };
 
 static void enable_dev(struct device *dev)
 {
-	pnp_enable_devices(dev, &pnp_ops,
+	pnp_enable_devices(dev, &ops,
 		sizeof(pnp_dev_info)/sizeof(pnp_dev_info[0]), pnp_dev_info);
 }
 
